package cn.wustlinghang.wusthelper.main.response;


/**
 * 通用的响应
 *
 * @param code 响应码
 * @param msg  响应信息
 * @param data 响应数据
 * @param <T>  data的类型
 */
public record Response<T>(int code, String msg, T data) {
    public static <T> Response<T> success(T data) {
        return new Response<>(ResponseCode.SUCCESS.getCode(), "ok", data);
    }

    public static <T> Response<T> success() {
        return success(null);
    }

    public static <T> Response<T> error(int code, String message) {
        return new Response<>(code, message, null);
    }

    public static <T> Response<T> error(ResponseCode code) {
        return error(code.getCode(), code.getMessage());
    }
}