package cn.wustlinghang.wusthelper.internal.rpc.client;

import cn.wustlinghang.mywust.network.Requester;
import cn.wustlinghang.mywust.network.entitys.HttpResponse;
import cn.wustlinghang.mywust.network.request.RequestFactory;
import cn.wustlinghang.wusthelper.internal.rpc.config.RegisterConfig;
import cn.wustlinghang.wusthelper.internal.rpc.entity.RegisterRequestBody;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.google.common.collect.Lists;
import lombok.extern.slf4j.Slf4j;

import java.io.IOException;

@Slf4j
public class ConsulClient {
    private final String serviceId;
    private final String serviceName;
    private final String consulAddress;

    private final ObjectMapper objectMapper;
    private final Requester requester;

    public ConsulClient(String serviceId, RegisterConfig registerConfig, ObjectMapper objectMapper, Requester requester) {
        this.serviceId = serviceId;
        this.serviceName = registerConfig.getServiceName();
        this.consulAddress = registerConfig.getConsulAddress();

        this.objectMapper = objectMapper;
        this.requester = requester;
    }

    public void register(String remoteAddress, int remotePort,
                         RegisterRequestBody.HealthCheckOption... healthCheckOption
    ) throws IOException {
        var registerRequestBody = RegisterRequestBody.builder()
                .id(serviceId)
                .name(serviceName)
                .address(remoteAddress)
                .port(remotePort)
                .checks(Lists.newArrayList(healthCheckOption))
                .build();

        byte[] registerRequestData = objectMapper.writeValueAsBytes(registerRequestBody);
        var url = consulAddress + "/v1/agent/service/register";

        var registerRequest = RequestFactory.makeHttpRequest(url, registerRequestData);
        var registerResponse = requester.put(registerRequest);

        if (registerResponse.getStatusCode() != HttpResponse.HTTP_OK) {
            log.warn("注册中心注册不成功，请手动注册");
        }
    }

    public void deregister() throws IOException {
        var url = String.format("%s/v1/agent/service/deregister/%s", consulAddress, serviceId);
        var deregisterRequest = RequestFactory.makeHttpRequest(url);
        var deregisterResponse = requester.put(deregisterRequest);

        if (deregisterResponse.getStatusCode() != HttpResponse.HTTP_OK) {
            log.warn("服务注销不成功，请手动注册");
        }
    }
}
