package cn.wustlinghang.wusthelper.internal.rpc.client;

import cn.hutool.core.codec.Base64;
import cn.wustlinghang.mywust.network.request.RequestFactory;
import cn.wustlinghang.mywust.network.Requester;
import cn.wustlinghang.wusthelper.internal.rpc.config.FrpConfig;
import com.fasterxml.jackson.databind.JsonNode;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.databind.node.MissingNode;
import org.ini4j.Ini;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;

public class FrpcClient {
    private final String frpcAdminAddress;

    private final String frpAuthHeaderValue;

    private final Requester requester;

    private final ObjectMapper objectMapper;

    public FrpcClient(FrpConfig config, Requester requester, ObjectMapper objectMapper) {

        this.frpcAdminAddress = config.getFrpcAdminAddress();

        this.requester = requester;
        this.objectMapper = objectMapper;

        String username = config.getFrpcAdminUsername();
        String password = config.getFrpcAdminPassword();
        this.frpAuthHeaderValue = "Basic " + Base64.encode(username + ":" + password);
    }

    private String frpProxyName;

    public void addFrpProxy(String serviceName, String serviceId, String localAddress, String localPort)
            throws IOException {
        this.frpProxyName = String.format("%s-%s", serviceName, serviceId);

        Ini ini = this.getFrpConfig();
        ini.put(frpProxyName, "type", "tcp");
        ini.put(frpProxyName, "local_address", localAddress);
        ini.put(frpProxyName, "local_port", localPort);
        ini.put(frpProxyName, "use_compression", "true");

        var output = new ByteArrayOutputStream();
        ini.store(output);

        this.reloadFrpConfig(ini);
    }

    public void removeProxy() throws IOException {
        Ini ini = this.getFrpConfig();
        ini.remove(this.frpProxyName);

        this.reloadFrpConfig(ini);
    }

    public Ini getFrpConfig() throws IOException {
        var frpConfigRequest = RequestFactory.makeHttpRequest(frpcAdminAddress + "/api/config");
        frpConfigRequest.addHeaders("Authorization", frpAuthHeaderValue);
        var response = requester.get(frpConfigRequest);

        return new Ini(new ByteArrayInputStream(response.getBody()));
    }

    public void reloadFrpConfig(Ini ini) throws IOException {
        var output = new ByteArrayOutputStream();
        ini.store(output);

        var uploadRequest = RequestFactory.makeHttpRequest(frpcAdminAddress + "/api/config", output.toByteArray());
        uploadRequest.addHeaders("Authorization", frpAuthHeaderValue);
        requester.put(uploadRequest);
        var reloadRequest = RequestFactory.makeHttpRequest(frpcAdminAddress + "/api/reload");
        reloadRequest.addHeaders("Authorization", frpAuthHeaderValue);
        requester.get(reloadRequest);
    }

    public JsonNode getProxyStatus(String serviceName, String serviceId) throws IOException {
        var statusRequest = RequestFactory.makeHttpRequest(frpcAdminAddress + "/api/status");
        statusRequest.addHeaders("Authorization", frpAuthHeaderValue);
        var statusResponse = requester.get(statusRequest);

        JsonNode proxies = objectMapper.readTree(statusResponse.getBody()).path("tcp");
        for (JsonNode proxy : proxies) {
            if (proxy.path("name").asText().equals(serviceName + "-" + serviceId)) {
                return proxy;
            }
        }

        return MissingNode.getInstance();
    }
}
