package cn.wustlinghang.wusthelper.internal.graduate.api.http.v1.handler;

import cn.wustlinghang.wusthelper.internal.rpc.response.RpcCommonResponseCode;
import cn.wustlinghang.wusthelper.internal.rpc.response.RpcResponse;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import jakarta.ws.rs.core.MediaType;
import jakarta.ws.rs.core.Response;

public abstract class BaseExceptionHandler {
    public static final String EXCEPTION_HEADER_KEY = "X-exception-handler";

    private static final ObjectMapper objectMapper = new ObjectMapper();

    public Response toResponse(RpcCommonResponseCode code, String msg, String handlerName) {
        return toResponse(code.getCode(), msg, handlerName);
    }

    public Response toResponse(int code, String msg, String handlerName) {
        return toResponse(Response.Status.INTERNAL_SERVER_ERROR.getStatusCode(), code, msg, handlerName);
    }

    public Response toResponse(int status, int code, String msg, String handlerName) {
        return toResponse(Response.Status.fromStatusCode(status), code, msg, handlerName);
    }

    public Response toResponse(Response.Status status, int code, String msg, String handlerName) {
        Object response;
        try {
            response = objectMapper.writeValueAsString(RpcResponse.error(code, msg));
        } catch (JsonProcessingException e) {
            response = RpcResponse.error(RpcCommonResponseCode.SERVER_INTERNAL_ERROR);
        }

        return Response.status(status)
                .entity(response)
                .header(EXCEPTION_HEADER_KEY, handlerName)
                .type(MediaType.APPLICATION_JSON)
                .build();
    }
}
