package cn.wustlinghang.wusthelper.internal.graduate.services;

import cn.wustlinghang.mywust.core.parser.graduate.GraduateStudentInfoPageParser;
import cn.wustlinghang.mywust.core.request.service.graduate.GraduateStudentInfoApiService;
import cn.wustlinghang.mywust.data.global.StudentInfo;
import cn.wustlinghang.mywust.exception.ApiException;
import cn.wustlinghang.mywust.network.RequestClientOption;
import cn.wustlinghang.wusthelper.internal.rpc.exception.GraduateRpcException;
import cn.wustlinghang.wusthelper.internal.rpc.exception.RpcException;
import jakarta.enterprise.context.ApplicationScoped;

import java.io.IOException;

@ApplicationScoped
public class StudentInfoService extends BaseService {

    private final RequestClientOption requestClientOption;

    private final GraduateStudentInfoApiService studentInfoApiService;
    private final GraduateStudentInfoPageParser studentInfoPageParser;

    public StudentInfoService(RequestClientOption requestClientOption,
                              GraduateStudentInfoApiService studentInfoApiService,
                              GraduateStudentInfoPageParser studentInfoPageParser) {
        this.requestClientOption = requestClientOption;
        
        this.studentInfoApiService = studentInfoApiService;
        this.studentInfoPageParser = studentInfoPageParser;
    }

    public String getStudentInfoPage(String cookie) throws RpcException {
        try {
            return studentInfoApiService.getPage(cookie, requestClientOption);
        } catch (ApiException e) {
            throw wrapApiException(e, GraduateRpcException.SubModuleCode.STUDENT_INFO);
        } catch (IOException e) {
            throw new GraduateRpcException(
                    GraduateRpcException.TypeCode.NETWORK_EXCEPTION,
                    GraduateRpcException.SubModuleCode.STUDENT_INFO,
                    GraduateRpcException.ErrorCode.NETWORK_ERROR
            );
        }
    }

    public StudentInfo parseStudentInfo(String data) throws RpcException {
        try {
            return studentInfoPageParser.parse(data);
        } catch (Exception e) {
            throw new GraduateRpcException(
                    GraduateRpcException.TypeCode.PARSE_EXCEPTION,
                    GraduateRpcException.SubModuleCode.STUDENT_INFO,
                    GraduateRpcException.ErrorCode.PARSE_ERROR
            );
        }
    }
}
