package cn.wustlinghang.wusthelper.internal.graduate.services;

import cn.wustlinghang.mywust.core.parser.graduate.GraduateTrainingPlanPageParser;
import cn.wustlinghang.mywust.core.request.service.graduate.GraduateTrainingPlanApiService;
import cn.wustlinghang.mywust.exception.ApiException;
import cn.wustlinghang.mywust.network.RequestClientOption;
import cn.wustlinghang.wusthelper.internal.rpc.exception.GraduateRpcException;
import cn.wustlinghang.wusthelper.internal.rpc.exception.RpcException;
import jakarta.enterprise.context.ApplicationScoped;

import java.io.IOException;

@ApplicationScoped
public class TrainingPlanService extends BaseService {

    private final RequestClientOption requestClientOption;

    private final GraduateTrainingPlanApiService trainingPlanApiService;
    private final GraduateTrainingPlanPageParser trainingPlanPageParser;

    public TrainingPlanService(RequestClientOption requestClientOption,
                               GraduateTrainingPlanApiService trainingPlanApiService,
                               GraduateTrainingPlanPageParser trainingPlanPageParser) {
        this.requestClientOption = requestClientOption;

        this.trainingPlanApiService = trainingPlanApiService;
        this.trainingPlanPageParser = trainingPlanPageParser;
    }

    public String getTrainingPlan(String cookie) throws RpcException {
        try {
            return trainingPlanApiService.getPage(cookie, requestClientOption);
        } catch (ApiException e) {
            throw wrapApiException(e, GraduateRpcException.SubModuleCode.TRAINING_PLAN);
        } catch (IOException e) {
            throw new GraduateRpcException(
                    GraduateRpcException.TypeCode.NETWORK_EXCEPTION,
                    GraduateRpcException.SubModuleCode.TRAINING_PLAN,
                    GraduateRpcException.ErrorCode.NETWORK_ERROR
            );
        }
    }

    public String parseTrainingPlan(String data) throws RpcException {
        try {
            return trainingPlanPageParser.parse(data);
        } catch (Exception e) {
            throw new GraduateRpcException(
                    GraduateRpcException.TypeCode.PARSE_EXCEPTION,
                    GraduateRpcException.SubModuleCode.TRAINING_PLAN,
                    GraduateRpcException.ErrorCode.PARSE_ERROR
            );
        }
    }
}
