package cn.wustlinghang.wusthelper.internal.library.services;

import cn.wustlinghang.mywust.core.request.service.library.CurrentLoanApiService;
import cn.wustlinghang.mywust.data.library.PagingResult;
import cn.wustlinghang.mywust.data.library.origin.CurrentLoanBook;
import cn.wustlinghang.mywust.exception.ApiException;
import cn.wustlinghang.mywust.exception.ParseException;
import cn.wustlinghang.wusthelper.internal.rpc.exception.LibraryRpcException;
import cn.wustlinghang.wusthelper.internal.rpc.exception.RpcException;
import jakarta.enterprise.context.ApplicationScoped;

import java.io.IOException;
import java.util.List;

@ApplicationScoped
public class CurrentLoanService extends BaseService {
    private final CurrentLoanApiService api;

    public CurrentLoanService(CurrentLoanApiService api) {
        this.api = api;
    }

    public PagingResult<List<CurrentLoanBook>> getCurrentLoan(String cookie, int page, int pageSize)
            throws RpcException {
        try {
            return api.getCurrentLoan(cookie, page, pageSize);
        } catch (ApiException e) {
            throw wrapApiException(e, LibraryRpcException.SubModuleCode.CURRENT_LOAN);
        } catch (IOException e) {
            throw new LibraryRpcException(
                    LibraryRpcException.TypeCode.NETWORK_EXCEPTION,
                    LibraryRpcException.SubModuleCode.CURRENT_LOAN,
                    LibraryRpcException.ErrorCode.NETWORK_ERROR
            );
        } catch (ParseException e) {
            throw new LibraryRpcException(
                    LibraryRpcException.TypeCode.PARSE_EXCEPTION,
                    LibraryRpcException.SubModuleCode.CURRENT_LOAN,
                    LibraryRpcException.ErrorCode.PARSE_ERROR
            );
        }
    }
}
