package cn.wustlinghang.wusthelper.internal.library.services;

import cn.wustlinghang.mywust.core.request.service.library.LoanHistoryApiService;
import cn.wustlinghang.mywust.data.library.PagingResult;
import cn.wustlinghang.mywust.data.library.origin.HistoryLoanBook;
import cn.wustlinghang.mywust.exception.ApiException;
import cn.wustlinghang.mywust.exception.ParseException;
import cn.wustlinghang.wusthelper.internal.rpc.exception.LibraryRpcException;
import cn.wustlinghang.wusthelper.internal.rpc.exception.RpcException;
import jakarta.enterprise.context.ApplicationScoped;
import lombok.extern.slf4j.Slf4j;

import java.io.IOException;
import java.util.List;

@Slf4j
@ApplicationScoped
public class LoanHistoryService extends BaseService {
    private final LoanHistoryApiService api;

    public LoanHistoryService(LoanHistoryApiService api) {
        this.api = api;
    }

    public PagingResult<List<HistoryLoanBook>> getLoanHistory(String cookie, int page, int pageSize)
            throws RpcException {
        try {
            return api.getLoanHistory(cookie, page, pageSize);
        } catch (ApiException e) {
            throw wrapApiException(e, LibraryRpcException.SubModuleCode.LOAN_HISTORY);
        } catch (IOException e) {
            throw new LibraryRpcException(
                    LibraryRpcException.TypeCode.NETWORK_EXCEPTION,
                    LibraryRpcException.SubModuleCode.LOAN_HISTORY,
                    LibraryRpcException.ErrorCode.NETWORK_ERROR
            );
        } catch (ParseException e) {
            log.error("解析错误：", e);
            throw new LibraryRpcException(
                    LibraryRpcException.TypeCode.PARSE_EXCEPTION,
                    LibraryRpcException.SubModuleCode.LOAN_HISTORY,
                    LibraryRpcException.ErrorCode.PARSE_ERROR
            );
        }
    }
}
