package cn.wustlinghang.wusthelper.internal.library.services;

import cn.wustlinghang.mywust.core.request.service.library.OverdueSoonApiService;
import cn.wustlinghang.mywust.data.library.PagingResult;
import cn.wustlinghang.mywust.data.library.origin.BaseLoanBook;
import cn.wustlinghang.mywust.exception.ApiException;
import cn.wustlinghang.mywust.exception.ParseException;
import cn.wustlinghang.wusthelper.internal.rpc.exception.LibraryRpcException;
import cn.wustlinghang.wusthelper.internal.rpc.exception.RpcException;
import jakarta.enterprise.context.ApplicationScoped;

import java.io.IOException;
import java.util.List;

@ApplicationScoped
public class OverdueSoonService extends BaseService {
    private final OverdueSoonApiService api;

    public OverdueSoonService(OverdueSoonApiService api) {
        this.api = api;
    }

    public PagingResult<List<BaseLoanBook>> getOverdueSoon(String cookie, int page, int pageSize)
            throws RpcException {
        try {
            return api.getOverdueSoon(cookie, page, pageSize);
        } catch (ApiException e) {
            throw wrapApiException(e, LibraryRpcException.SubModuleCode.OVERDUE_SOON);
        } catch (IOException e) {
            throw new LibraryRpcException(
                    LibraryRpcException.TypeCode.NETWORK_EXCEPTION,
                    LibraryRpcException.SubModuleCode.OVERDUE_SOON,
                    LibraryRpcException.ErrorCode.NETWORK_ERROR
            );
        } catch (ParseException e) {
            throw new LibraryRpcException(
                    LibraryRpcException.TypeCode.PARSE_EXCEPTION,
                    LibraryRpcException.SubModuleCode.OVERDUE_SOON,
                    LibraryRpcException.ErrorCode.PARSE_ERROR
            );
        }
    }
}
