package cn.wustlinghang.wusthelper.internal.library.services;

import cn.wustlinghang.mywust.core.request.service.library.SearchApiService;
import cn.wustlinghang.mywust.data.library.PagingResult;
import cn.wustlinghang.mywust.data.library.origin.BookSearchResult;
import cn.wustlinghang.mywust.exception.ApiException;
import cn.wustlinghang.mywust.exception.ParseException;
import cn.wustlinghang.wusthelper.internal.rpc.exception.LibraryRpcException;
import cn.wustlinghang.wusthelper.internal.rpc.exception.RpcException;
import jakarta.enterprise.context.ApplicationScoped;

import java.io.IOException;
import java.util.List;

@ApplicationScoped
public class SearchService extends BaseService {
    private final SearchApiService api;

    public SearchService(SearchApiService api) {
        this.api = api;
    }

    public PagingResult<List<BookSearchResult>> search(String keyword, int page, int pageSize)
            throws RpcException {
        try {
            return api.search(keyword, page, pageSize);
        } catch (ApiException e) {
            throw wrapApiException(e, LibraryRpcException.SubModuleCode.SEARCH);
        } catch (IOException e) {
            throw new LibraryRpcException(
                    LibraryRpcException.TypeCode.NETWORK_EXCEPTION,
                    LibraryRpcException.SubModuleCode.SEARCH,
                    LibraryRpcException.ErrorCode.NETWORK_ERROR
            );
        } catch (ParseException e) {
            throw new LibraryRpcException(
                    LibraryRpcException.TypeCode.PARSE_EXCEPTION,
                    LibraryRpcException.SubModuleCode.SEARCH,
                    LibraryRpcException.ErrorCode.PARSE_ERROR
            );
        }
    }
}