package cn.wustlinghang.wusthelper.internal.physics.services;

import cn.wustlinghang.mywust.core.parser.physics.PhysicsCoursePageParser;
import cn.wustlinghang.mywust.core.request.service.physics.PhysicsCourseApiService;
import cn.wustlinghang.mywust.data.physics.PhysicsCourse;
import cn.wustlinghang.mywust.exception.ApiException;
import cn.wustlinghang.mywust.network.RequestClientOption;
import cn.wustlinghang.wusthelper.internal.rpc.exception.PhysicsRpcException;
import cn.wustlinghang.wusthelper.internal.rpc.exception.RpcException;
import jakarta.enterprise.context.ApplicationScoped;

import java.io.IOException;
import java.util.List;

@ApplicationScoped
public class CourseTableService extends BaseService {

    private final RequestClientOption requestClientOption;

    private final PhysicsCourseApiService agent;
    private final PhysicsCoursePageParser parser;

    public CourseTableService(RequestClientOption requestClientOption,
                              PhysicsCourseApiService agent,
                              PhysicsCoursePageParser parser) {
        this.requestClientOption = requestClientOption;

        this.agent = agent;
        this.parser = parser;
    }

    public String getCourseTable(String cookie) throws RpcException {
        try {
            return agent.getPage(cookie, requestClientOption);
        } catch (ApiException e) {
            throw wrapApiException(e, PhysicsRpcException.SubModuleCode.COURSE_TABLE);
        } catch (IOException e) {
            throw new PhysicsRpcException(
                    PhysicsRpcException.TypeCode.NETWORK_EXCEPTION,
                    PhysicsRpcException.SubModuleCode.COURSE_TABLE,
                    PhysicsRpcException.ErrorCode.NETWORK_ERROR
            );
        }
    }

    public List<PhysicsCourse> parseCourseTable(String data) throws PhysicsRpcException {
        try {
            return parser.parse(data);
        } catch (Exception e) {
            throw new PhysicsRpcException(
                    PhysicsRpcException.TypeCode.PARSE_EXCEPTION,
                    PhysicsRpcException.SubModuleCode.COURSE_TABLE,
                    PhysicsRpcException.ErrorCode.PARSE_ERROR
            );
        }
    }
}
