package cn.wustlinghang.wusthelper.internal.physics.services;

import cn.wustlinghang.mywust.core.parser.physics.PhysicsScorePageParser;
import cn.wustlinghang.mywust.core.request.service.physics.PhysicsScoreApiService;
import cn.wustlinghang.mywust.data.global.Score;
import cn.wustlinghang.mywust.exception.ApiException;
import cn.wustlinghang.mywust.exception.ParseException;
import cn.wustlinghang.mywust.network.RequestClientOption;
import cn.wustlinghang.wusthelper.internal.rpc.exception.PhysicsRpcException;
import cn.wustlinghang.wusthelper.internal.rpc.exception.RpcException;
import jakarta.enterprise.context.ApplicationScoped;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

@ApplicationScoped
public class ScoreService extends BaseService {

    private final RequestClientOption requestClientOption;

    private final PhysicsScoreApiService agent;
    private final PhysicsScorePageParser parser;

    public ScoreService(RequestClientOption requestClientOption,
                        PhysicsScoreApiService agent,
                        PhysicsScorePageParser parser) {
        this.requestClientOption = requestClientOption;

        this.agent = agent;
        this.parser = parser;
    }

    public List<String> getScore(String cookie) throws RpcException {
        try {
            return agent.getAllPages(cookie, requestClientOption);
        } catch (ApiException e) {
            throw wrapApiException(e, PhysicsRpcException.SubModuleCode.SCORE);
        } catch (IOException e) {
            throw new PhysicsRpcException(
                    PhysicsRpcException.TypeCode.NETWORK_EXCEPTION,
                    PhysicsRpcException.SubModuleCode.SCORE,
                    PhysicsRpcException.ErrorCode.NETWORK_ERROR
            );
        } catch (ParseException e) {
            throw new PhysicsRpcException(
                    PhysicsRpcException.TypeCode.OTHER_EXCEPTION,
                    PhysicsRpcException.SubModuleCode.SCORE,
                    PhysicsRpcException.ErrorCode.PARSE_ERROR
            );
        }
    }


    public List<Score> parseScore(String data) throws RpcException {
        try {
            return parser.parse(data);
        } catch (PhysicsRpcException e) {
            throw e;
        } catch (Exception e) {
            throw new PhysicsRpcException(
                    PhysicsRpcException.TypeCode.PARSE_EXCEPTION,
                    PhysicsRpcException.SubModuleCode.SCORE,
                    PhysicsRpcException.ErrorCode.PARSE_ERROR
            );
        }
    }

    public List<Score> parseScore(List<String> pages) throws RpcException {
        try {
            List<Score> scores = new ArrayList<>();
            for (String page : pages) {
                scores.addAll(parser.parse(page));
            }

            return scores;
        } catch (PhysicsRpcException e) {
            throw e;
        } catch (Exception e) {
            throw new PhysicsRpcException(
                    PhysicsRpcException.TypeCode.PARSE_EXCEPTION,
                    PhysicsRpcException.SubModuleCode.SCORE,
                    PhysicsRpcException.ErrorCode.PARSE_ERROR
            );
        }
    }
}
