package cn.wustlinghang.wusthelper.internal.undergrad.rpc.client;

import cn.wustlinghang.mywust.network.request.RequestFactory;
import cn.wustlinghang.mywust.network.Requester;
import cn.wustlinghang.mywust.network.entitys.HttpResponse;
import cn.wustlinghang.wusthelper.internal.undergrad.rpc.entity.RegisterRequestBody;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.google.common.collect.Lists;
import lombok.extern.slf4j.Slf4j;

import java.io.IOException;

@Slf4j
public class ConsulClient {
    private final String serviceId;
    private final String serviceName;
    private final String consulAddress;
    private final ObjectMapper objectMapper;
    private final Requester requester;

    public ConsulClient(String serviceName, String serviceId, String consulAddress,
                        ObjectMapper objectMapper, Requester requester) {
        this.serviceId = serviceId;
        this.serviceName = serviceName;
        this.consulAddress = consulAddress;

        this.objectMapper = objectMapper;
        this.requester = requester;
    }

    public void register(String remoteAddress, int remotePort) throws IOException {
        var checksItem = RegisterRequestBody.ChecksItem.builder()
                .http("http://" + remoteAddress + "/health")
                .interval("3s")
                .build();

        var registerRequestBody = RegisterRequestBody.builder()
                .id(serviceId)
                .name(serviceName)
                .address("127.0.0.1")
                .port(remotePort)
                .checks(Lists.newArrayList(checksItem))
                .build();

        byte[] registerRequestData = objectMapper.writeValueAsBytes(registerRequestBody);
        var registerRequest = RequestFactory.makeHttpRequest(consulAddress + "/v1/agent/service/register", registerRequestData);
        var registerResponse = requester.put(registerRequest);
        if (registerResponse.getStatusCode() != HttpResponse.HTTP_OK) {
            log.warn("注册中心注册不成功，请手动注册");
        }
    }

    public void deregister() throws IOException {
        var deregisterRequest = RequestFactory.makeHttpRequest(consulAddress + "/v1/agent/service/deregister/" + serviceId);
        var deregisterResponse = requester.put(deregisterRequest);
        if (deregisterResponse.getStatusCode() != HttpResponse.HTTP_OK) {
            log.warn("服务注销不成功，请手动注册");
        }
    }
}
