package cn.wustlinghang.wusthelper.internal.undergrad.services;

import cn.wustlinghang.mywust.core.parser.undergraduate.UndergradCreditStatusParser;
import cn.wustlinghang.mywust.core.request.service.undergraduate.UndergradCreditStatusApiService;
import cn.wustlinghang.mywust.exception.ApiException;
import cn.wustlinghang.mywust.network.RequestClientOption;
import cn.wustlinghang.wusthelper.internal.rpc.exception.UndergradRpcException;
import cn.wustlinghang.wusthelper.internal.rpc.exception.RpcException;
import jakarta.enterprise.context.ApplicationScoped;

import java.io.IOException;

@ApplicationScoped
public class CreditStatusService extends BaseService {
    private final UndergradCreditStatusApiService creditStatusApiService;
    private final UndergradCreditStatusParser creditStatusParser;

    private final RequestClientOption requestClientOption;
    public CreditStatusService(UndergradCreditStatusApiService creditStatusApiService,
                               UndergradCreditStatusParser creditStatusParser,
                               RequestClientOption requestClientOption) {
        this.creditStatusApiService = creditStatusApiService;
        this.creditStatusParser = creditStatusParser;

        this.requestClientOption = requestClientOption;
    }

    public String getCreditStatus(String cookie) throws RpcException {
        try {
            return creditStatusApiService.getPage(cookie, requestClientOption, false);
        } catch (ApiException e) {
            throw wrapApiException(e, UndergradRpcException.SubModuleCode.CREDIT_STATUS);
        } catch (IOException e) {
            throw new UndergradRpcException(
                    UndergradRpcException.TypeCode.NETWORK_EXCEPTION,
                    UndergradRpcException.SubModuleCode.CREDIT_STATUS,
                    UndergradRpcException.ErrorCode.NETWORK_ERROR
            );
        }
    }

    public String parseCreditStatus(String data) throws RpcException {
        try {
            return creditStatusParser.parse(data);
        } catch (Exception e) {
            throw new UndergradRpcException(
                    UndergradRpcException.TypeCode.PARSE_EXCEPTION,
                    UndergradRpcException.SubModuleCode.CREDIT_STATUS,
                    UndergradRpcException.ErrorCode.PARSE_ERROR
            );
        }
    }
}
