package cn.wustlinghang.wusthelper.internal.undergrad.services;

import cn.wustlinghang.mywust.core.request.service.auth.UndergraduateLogin;
import cn.wustlinghang.mywust.exception.ApiException;
import cn.wustlinghang.mywust.network.RequestClientOption;
import cn.wustlinghang.wusthelper.internal.rpc.exception.UndergradRpcException;
import jakarta.enterprise.context.ApplicationScoped;
import lombok.extern.slf4j.Slf4j;

import java.io.IOException;

@Slf4j
@ApplicationScoped
public class LoginService {
    private final RequestClientOption option;
    private final UndergraduateLogin undergraduateLogin;

    public LoginService(RequestClientOption option,
                        UndergraduateLogin undergraduateLogin) {
        this.option = option;
        this.undergraduateLogin = undergraduateLogin;
    }

    public String login(String username, String password, boolean legacy) throws UndergradRpcException {
        try {
            if (legacy) {
                return undergraduateLogin.getLoginCookieLegacy(username, password, option);
            } else {
                return undergraduateLogin.getLoginCookie(username, password, option);
            }
        } catch (ApiException e) {
            throw wrapApiException(e);
        } catch (IOException e) {
            throw new UndergradRpcException(
                    UndergradRpcException.TypeCode.NETWORK_EXCEPTION,
                    UndergradRpcException.SubModuleCode.AUTH,
                    UndergradRpcException.ErrorCode.NETWORK_ERROR
            );
        }
    }

    public boolean verify(String cookie) throws UndergradRpcException {
        try {
            return undergraduateLogin.testCookie(cookie);
        } catch (ApiException e) {
            return false;
        } catch (IOException e) {
            throw new UndergradRpcException(
                    UndergradRpcException.TypeCode.NETWORK_EXCEPTION,
                    UndergradRpcException.SubModuleCode.AUTH,
                    UndergradRpcException.ErrorCode.NETWORK_ERROR
            );
        }
    }

    private UndergradRpcException wrapApiException(ApiException e) {
        UndergradRpcException.ErrorCode errorCode = switch (e.getCode()) {
            case NETWORK_EXCEPTION -> UndergradRpcException.ErrorCode.NETWORK_ERROR;
            case UNI_LOGIN_PASSWORD_WRONG -> UndergradRpcException.ErrorCode.AUTH_PASSWORD_WRONG;
            case UNI_LOGIN_USER_NOT_EXISTS -> UndergradRpcException.ErrorCode.AUTH_USER_NOT_EXISTS;
            case UNI_LOGIN_USER_BANNED -> UndergradRpcException.ErrorCode.AUTH_USER_BANNED;
            case UNI_LOGIN_USER_DISABLED -> UndergradRpcException.ErrorCode.AUTH_USER_DISABLED;
            case UNI_LOGIN_NEED_CHANGE_PASSWORD -> UndergradRpcException.ErrorCode.AUTH_NEED_CHANGE_PASSWORD;
            case UNDERGRAD_BANNED_IN_EXCLUSIVE_TIME -> UndergradRpcException.ErrorCode.AUTH_BANNED_IN_EXCLUSIVE_TIME;
            case UNI_LOGIN_USER_NOT_ONLY, UNI_LOGIN_NO_REGISTER, UNI_LOGIN_NEED_TFA
                    -> UndergradRpcException.ErrorCode.AUTH_UNKNOWN_ERROR;
            default -> {
                log.error("本科生：登录代理请求异常，异常未处理");
                log.error("异常：", e);
                yield UndergradRpcException.ErrorCode.AUTH_UNKNOWN_ERROR;
            }
        };

        return new UndergradRpcException(
                UndergradRpcException.TypeCode.AUTH_EXCEPTION,
                UndergradRpcException.SubModuleCode.AUTH,
                errorCode
        );
    }
}
