package cn.wustlinghang.wusthelper.internal.undergrad.services;

import cn.wustlinghang.mywust.core.parser.undergraduate.UndergradScoreParser;
import cn.wustlinghang.mywust.core.request.service.undergraduate.UndergradScoreApiService;
import cn.wustlinghang.mywust.data.global.Score;
import cn.wustlinghang.mywust.exception.ApiException;
import cn.wustlinghang.wusthelper.internal.rpc.exception.UndergradRpcException;
import cn.wustlinghang.wusthelper.internal.rpc.exception.RpcException;
import jakarta.enterprise.context.ApplicationScoped;

import java.io.IOException;
import java.util.List;

@ApplicationScoped
public class ScoreService extends BaseService {
    private final UndergradScoreApiService scoreApiService;
    private final UndergradScoreParser scoreParser;

    public ScoreService(UndergradScoreApiService scoreApiService,
                        UndergradScoreParser scoreParser) {
        this.scoreApiService = scoreApiService;
        this.scoreParser = scoreParser;
    }

    public String getScore(String cookie) throws RpcException {
        try {
            return scoreApiService.getPage(cookie);
        } catch (ApiException e) {
            throw wrapApiException(e, UndergradRpcException.SubModuleCode.SCORE);
        } catch (IOException e) {
            throw new UndergradRpcException(
                    UndergradRpcException.TypeCode.NETWORK_EXCEPTION,
                    UndergradRpcException.SubModuleCode.SCORE,
                    UndergradRpcException.ErrorCode.NETWORK_ERROR
            );
        }
    }

    public List<Score> parseScore(String data) throws RpcException {
        try {
            if (data.contains("评教")) {
                throw new UndergradRpcException(
                        UndergradRpcException.TypeCode.PARSE_EXCEPTION,
                        UndergradRpcException.SubModuleCode.SCORE,
                        UndergradRpcException.ErrorCode.NEED_EVALUATE
                );
            }

            return scoreParser.parse(data);
        } catch (Exception e) {
            throw new UndergradRpcException(
                    UndergradRpcException.TypeCode.PARSE_EXCEPTION,
                    UndergradRpcException.SubModuleCode.SCORE,
                    UndergradRpcException.ErrorCode.PARSE_ERROR
            );
        }
    }
}
