package cn.wustlinghang.wusthelper.internal.undergrad.services;

import cn.wustlinghang.mywust.core.parser.undergraduate.UndergradTrainingPlanPageParser;
import cn.wustlinghang.mywust.core.request.service.undergraduate.UndergradTrainingPlanApiService;
import cn.wustlinghang.mywust.exception.ApiException;
import cn.wustlinghang.wusthelper.internal.rpc.exception.UndergradRpcException;
import cn.wustlinghang.wusthelper.internal.rpc.exception.RpcException;
import jakarta.enterprise.context.ApplicationScoped;

import java.io.IOException;

@ApplicationScoped
public class TrainingPlanService extends BaseService {
    private final UndergradTrainingPlanApiService trainingPlanApiService;
    private final UndergradTrainingPlanPageParser trainingPlanPageParser;

    public TrainingPlanService(UndergradTrainingPlanApiService trainingPlanApiService,
                               UndergradTrainingPlanPageParser trainingPlanPageParser) {
        this.trainingPlanApiService = trainingPlanApiService;
        this.trainingPlanPageParser = trainingPlanPageParser;
    }

    public String getTrainingPlan(String cookie) throws RpcException {
        try {
            return trainingPlanApiService.getPage(cookie);
        } catch (ApiException e) {
            throw wrapApiException(e, UndergradRpcException.SubModuleCode.TRAINING_PLAN);
        } catch (IOException e) {
            throw new UndergradRpcException(
                    UndergradRpcException.TypeCode.NETWORK_EXCEPTION,
                    UndergradRpcException.SubModuleCode.TRAINING_PLAN,
                    UndergradRpcException.ErrorCode.NETWORK_ERROR
            );
        }
    }

    public String parseTrainingPlan(String data) throws RpcException {
        try {
            return trainingPlanPageParser.parse(data);
        } catch (Exception e) {
            throw new UndergradRpcException(
                    UndergradRpcException.TypeCode.PARSE_EXCEPTION,
                    UndergradRpcException.SubModuleCode.TRAINING_PLAN,
                    UndergradRpcException.ErrorCode.PARSE_ERROR
            );
        }
    }
}
